

// Include files for error codes and error messages
#include <stderror.mqh>
#include <stdlib.mqh>


// User Set-able Variables
extern double  dLots          =  1.0;
extern int     iMaxOrders     =  1;

extern double  iTakeProfit    =  50;
extern double  iStopLoss      =  50;
extern double  iTrailingStop  =  0;

extern int     iSlippage      =  3;

extern int iMagicNumber=1000;


// Trade Signal Variables
int iTradeSignal=0;
int iOpenBuySignal=100;
int iCloseBuySignal=-200;
int iOpenSellSignal=300;
int iCloseSellSignal=-400;
int iNoSignal=-1;

// Open Trade Information Variables
int iLastTradeType;
int iTotalTrades;
double dTotalTradeLots;
double dTotalTradeSwap;
int iTotalBuyTrades;
int iTotalSellTrades;
double dLastTradeOpenPrice;
int iLastTradeOpenTime;
double dLastTradeStopLoss;
double dLastTradeTakeProfit;
double dLastTradeType;
double dLastTradeTicket;
string strLastTradeSymbol;
string strLastTradeComment;
double dTotalOpenProfit;
int iTotalSelectedTrades;

// The init function, only execute the first time the Expert Advisor is attached to a chart
int init()
{
   iLastTradeType=1;
}
// -------------------------------------------------
// expert deinitialization function                                 
// -------------------------------------------------
int deinit()
  {
   return(0);
  }
  
// -------------------------------------------------
// expert global variables
// -------------------------------------------------

double fractalv_lower, fractalv_upper;
 
double demarkerv[], wprv[];
// The entry point of the Expert Advisor: the start function
int start()
{
   fnGetOpenTradeInfo();
   fnOpenBuyLogic();
   fnOpenSellLogic();
   fnCloseBuyLogic();
   fnCloseSellLogic();
   fnOrder();
   fnTrailingStop();
   
   return(0);
}

// The user-defined functions



// Get information about all open trades
void fnGetOpenTradeInfo()
{
    dTotalOpenProfit=0;
    dTotalTradeLots=0;
    iTotalBuyTrades=0;
    iTotalSellTrades=0;
    dTotalTradeSwap=0;
    iTotalSelectedTrades=0;
    iTotalTrades=OrdersTotal();
    for(int pos=0;pos<iTotalTrades;pos++)
    {
        if(OrderSelect(pos,SELECT_BY_POS)==false)
            continue;
        if( (( OrderSymbol() == Symbol() ) ) && (( OrderMagicNumber() == iMagicNumber ) ) )
			{

            iTotalSelectedTrades++;
            dTotalOpenProfit += OrderProfit();
            dTotalTradeSwap += OrderSwap();
            dTotalTradeLots += OrderLots();
            if( OrderType() == OP_BUY )
            {   
                iTotalBuyTrades++;
                iLastTradeType = OP_BUY;
            }
            if( OrderType() == OP_SELL )  
            {
                iTotalSellTrades++;
                iLastTradeType = OP_SELL;
            }
        }
    }
}
// -------------------------------------------------
// IsFractalLower()
// -------------------------------------------------
bool IsFractalLower()
  {
   return(fractalv_lower != 0); 
   //&&(fractalv_lower < gatorv_jaw || fractalv_lower > gatorv_lips));
      //fractalv_lower > gatorv_teeth));  /*** NEW ***/
     // && fractalv_upper == 0);
  } 
  
// -------------------------------------------------
// IsFractalUpper()
// -------------------------------------------------
bool IsFractalUpper()
  {
   return(fractalv_upper != 0);
   // && (fractalv_upper > gatorv_lips || fractalv_upper < gatorv_jaw));
         //fractalv_upper < gatorv_teeth));  /*** NEW ***/ 
   
   // && fractalv_lower == 0); 
  }      
     
// -------------------------------------------------
//  IsOrderProfitable 
// -------------------------------------------------
bool IsOrderProfitable() 
  {
   return(true); // TODO : analyze profitability
  }  
    
// -------------------------------------------------
// WasDemarkerLow()
// -------------------------------------------------
bool WasDemarkerLow()
  {
   return(ArrayMinValue(demarkerv) < 0.5); 
  } 
 
// -------------------------------------------------
// WasDemarkerHigh()
// -------------------------------------------------
bool WasDemarkerHigh()
  {
   return(ArrayMaxValue(demarkerv) > 0.5); 
  } 
  
// -------------------------------------------------
// WasWPROverBuy()
// -------------------------------------------------
bool WasWPROverBuy()
  {
   return(ArrayMinValue(wprv) <= 0.25); 
  }      
      
// -------------------------------------------------
// WasWPROverSell()
// -------------------------------------------------
bool WasWPROverSell()
  {
   return(ArrayMaxValue(wprv) >= 0.75); 
  } 
// -------------------------------------------------
// bool PrepareIndicators()
// -------------------------------------------------
bool PrepareIndicators()
  {
   int ip, ib, lookback; 
   double dm, fr;
   lookback = 3; // look for fresh markers only
   
   fractalv_lower = 0;
   fractalv_upper = 0;
      
   for(ib=0; ib<=lookback; ib++)
     {
      // Lower fractals 
      fr = iFractals(NULL, 0, MODE_LOWER, ib);
      if(IsError("Lower Fractals")) return(false);
      if(fr != 0) fractalv_lower = fr;
 
      // Upper fractals 
      fr = iFractals(NULL, 0, MODE_UPPER, ib);
      if(IsError("Upper Fractals")) return(false);
      if(fr != 0) fractalv_upper = fr;
     }
 
   // DeMArker and WPR
 
   lookback = 0;
 
   ArrayResize(demarkerv, lookback+1);
   ArrayResize(wprv, lookback+1);
 
   for(ib=0; ib<=lookback; ib++)
     {
      // DeMarker 
      demarkerv[ib] = iDeMarker(NULL, 0, 75, ib);
      if(IsError("DeMarker")) return(false);
 
      // William's Percent Rate * -1
      wprv[ib] = -iWPR(NULL, 0, 8, ib+1) / 100; 
      if(IsError("WPR")) return(false);
     }
   
   return(true);
  }

// The logic for opening a Buy order.
// This will set the varaiable iTradeSignal to iOpenBuySignal if the logic evaluates to true.
void fnOpenBuyLogic()
{   
   double dMovingAvg=0;
   dMovingAvg = iMA(NULL,NULL,12,0,MODE_EMA,PRICE_CLOSE,0);

   double dRelativeStrength =0;
   dRelativeStrength  = iRSI(NULL,NULL,12,PRICE_CLOSE,0);

   if(IsFractalLower() && WasDemarkerHigh()
   {
      if( (iLastTradeType == OP_SELL) || (iLastTradeType == -1) )
         iTradeSignal = iOpenBuySignal;
   } 
   
}

// The logic for opening a Sell order.
// This will set the varaiable iTradeSignal to iOpenSellSignal if the logic evaluates to true.
void fnOpenSellLogic()
{
   int iLowestIndex = iLowest(NULL,NULL,MODE_LOW,18,0);
   double dLowestValue = Low[iLowestIndex];
   
   if(IsFractalUpper() && WasDemarkerLow()
   {
      if( (iLastTradeType == OP_BUY) || (iLastTradeType == -1) )
         iTradeSignal = iOpenSellSignal;
   } 
}


// The logic for closing a Buy order.
// This will set the varaiable iTradeSignal to iCloseBuySignal if the logic evaluates to true.
void fnCloseBuyLogic()
{
   double dRelativeStrength =0;
   dRelativeStrength  = iRSI(NULL,NULL,12,PRICE_CLOSE,0);

   if( (dRelativeStrength < 40.0) )
   {
      iTradeSignal = iCloseBuySignal;
   } 
   
}

// The logic for closing a Sell order.
// This will set the varaiable iTradeSignal to iCloseSellSignal if the logic evaluates to true.
void fnCloseSellLogic()
{
   int iHighestIndex = iHighest(NULL,NULL,MODE_HIGH,18,0);
   double dHighestValue = Low[iHighestIndex];
   
   if( Bid > dHighestValue )
   {
      iTradeSignal = iCloseSellSignal;
   } 
}

// The code for opening or closing orders.
// This will execute according to the value of the varaiable iTradeSignal.
void fnOrder()
{
   int iOrderOpenStatus;
   string strErrorMessage;
   int iErrorNumber;
   bool bOrderCloseStatus;
   
   
   if( (iTradeSignal == iOpenBuySignal) && (iTotalSelectedTrades < iMaxOrders) )
   {
      double dBuyStopLoss = Ask-(iStopLoss*Point);
      double dBuyTakeProfit = Ask+(iTakeProfit*Point);
      iOrderOpenStatus = OrderSend(Symbol(), OP_BUY,dLots,Ask,iSlippage,dBuyStopLoss,dBuyTakeProfit,"iExpertAdvisor",iMagicNumber,0,Black);
      if( iOrderOpenStatus < 0 )
      {
        iErrorNumber = GetLastError();
        strErrorMessage = ErrorDescription(iErrorNumber);
        Print( "OrderSend Failed: ", strErrorMessage );
      } // end of if iOrderOpenStatus
      return;
   } // end of if iOpenBuySignal
   
   if( (iTradeSignal == iOpenSellSignal) && (iTotalSelectedTrades < iMaxOrders) )
   {
      double dSellStopLoss = Bid+(iStopLoss*Point);
      double dSellTakeProfit = Bid-(iTakeProfit*Point);
      OrderSend(Symbol(),OP_SELL,dLots,Bid,iSlippage,dSellStopLoss,dSellTakeProfit,"iExpertAdvisor",iMagicNumber,0,Blue);
      if( iOrderOpenStatus < 0 )
      {
        iErrorNumber = GetLastError();
        strErrorMessage = ErrorDescription(iErrorNumber);
        Print( "OrderSend Failed: ", strErrorMessage );
      } // end of if iOrderOpenStatus
      return;
   }	// end of if iOpenSellSignal				


   if( (iTradeSignal == iCloseBuySignal) || (iTradeSignal == iCloseBuySignal) )
   {
      iTotalTrades = OrdersTotal();
      for(int pos=0;pos<iTotalTrades;pos++)
      {
         if(OrderSelect(pos,SELECT_BY_POS)==false)
            continue;
            
         if( (( OrderSymbol() == Symbol() ) ) && (( OrderMagicNumber() == iMagicNumber ) ) )
         {
            if( OrderType() == OP_BUY )
            {
               bOrderCloseStatus = OrderClose( OrderTicket(), OrderLots(), Bid, iSlippage, Red );
               if( bOrderCloseStatus == false )
               {
                 iErrorNumber = GetLastError();
                 strErrorMessage = ErrorDescription(iErrorNumber);
                 Print( "OrderClose Failed: ", strErrorMessage );
                 return;
               } // end of if bOrderCloseStatus
            } // end of if OP_BUY
            
            if( OrderType() == OP_SELL )
            {
               bOrderCloseStatus = OrderClose( OrderTicket(), OrderLots(), Ask, iSlippage, Red );
               if( bOrderCloseStatus == false )
               {
                 iErrorNumber = GetLastError();
                 strErrorMessage = ErrorDescription(iErrorNumber);
                 Print( "OrderClose Failed: ", strErrorMessage );
                 return;
               } // end of if bOrderCloseStatus
            } // end of if OP_SELL
         } // end of if sysmbol
      } // end of OrderSelect for loop
   }  // end of if iCloseBuySignal
} // end of fnOrder

double dTrailingStop;

void fnTrailingStop()
{
   string strErrorMessage;
   int iErrorNumber;
   bool bOrderModifyStatus;
   
   if( iTrailingStop == 0 )
      return;
      
   iTotalTrades = OrdersTotal();
   for(int pos=0;pos<iTotalTrades;pos++)
   {
      if(OrderSelect(pos,SELECT_BY_POS)==false)
         continue;
      
      if( (( OrderSymbol() == Symbol() ) ) && (( OrderMagicNumber() == iMagicNumber ) ) )
      {
         if( OrderType() == OP_BUY )
         {
            dTrailingStop = Ask - (iTrailingStop *Point);
            if( (OrderProfit() > 0) && (OrderStopLoss() < dTrailingStop) )
            {
               bOrderModifyStatus = OrderModify(OrderTicket(),OrderOpenPrice(),dTrailingStop,OrderTakeProfit(), 0, Green);
               if( bOrderModifyStatus == false )
               {
                 iErrorNumber = GetLastError();
                 strErrorMessage = ErrorDescription(iErrorNumber);
                 Print( "OrderModify Failed: ", strErrorMessage );
                 return;
               } // end of if bOrderCloseStatus
            } // if OrderProfit
         } // end of if OP_BUY
      
         if( OrderType() == OP_SELL )
         {
            dTrailingStop = Bid + (iTrailingStop *Point);
            if( (OrderProfit() > 0) && (OrderStopLoss() > dTrailingStop) )
            {
               bOrderModifyStatus = OrderModify(OrderTicket(),OrderOpenPrice(),dTrailingStop,OrderTakeProfit(), 0, Green);
               if( bOrderModifyStatus == false )
               {
                 iErrorNumber = GetLastError();
                 strErrorMessage = ErrorDescription(iErrorNumber);
                 Print( "OrderModify Failed: ", strErrorMessage );
                 return;
               } // end of if bOrderCloseStatus
            } // end OrderModify
         } // end of if OP_SELL
      } // end of if sysmbol
   } // end of OrderSelect for loop
} // end fnTrailingStop

